﻿using anbs_cp.Interfaces;

namespace anbs_cp.Classes;

/// <summary>
/// Состояние действия
///
/// Обновлено 2023.11.1
/// * Добавлен возвращаемый результат
/// 
/// Обновлено 2023.01.121.1:
/// * Заменены интерфейсы IAction* на соответствующие классы
/// </summary>
public class ActionState<T>
{
    /// <summary>
    /// Конструктор по умолчанию
    /// </summary>
    public ActionState ()
    {
        Info = new();
        Warnings = new();
        Errors = new();
        Value = default;
    }

    /// <summary>
    /// Конструктор с указанием начального значения <see cref="Value"/>
    /// </summary>
    /// <param name="defaultValue">Начальное значение <see cref="Value"/></param>
    public ActionState (T defaultValue)
    {
        Info = new();
        Warnings = new();
        Errors = new();
        Value = defaultValue;
    }

    /// <summary>
    /// Список информации
    /// </summary>
    public List<ActionInfo> Info { get; }

    /// <summary>
    /// Список предупреждений
    /// </summary>
    public List<ActionWarning> Warnings { get; }

    /// <summary>
    /// Список ошибок
    /// </summary>
    public List<ActionError> Errors { get; }

    /// <summary>
    /// Значение
    /// </summary>
    public T? Value { get; set; }

    #region Методы

    #region Очистка
    /// <summary>
    /// Очищает список ошибок
    /// </summary>
    public void ClearErrors ()
    {
        Errors.Clear();
        Errors.TrimExcess();
    }

    /// <summary>
    /// Очищает список предупреждений
    /// </summary>
    public void ClearWarnings ()
    {
        Warnings.Clear();
        Warnings.TrimExcess();
    }

    /// <summary>
    /// Очищает список информации
    /// </summary>
    public void ClearInfo ()
    {
        Info.Clear();
        Info.TrimExcess();
    }

    /// <summary>
    /// Очищает все списки
    /// </summary>
    public void Clear ()
    {
        ClearInfo();
        ClearWarnings();
        ClearErrors();
    }
    #endregion

    #region Добавление ошибки
    /// <summary>
    /// Добавление ошибки
    /// </summary>
    /// <param name="error">Ошибка</param>
    // ReSharper disable once MemberCanBeMadeStatic.Global
    // ReSharper disable once FunctionRecursiveOnAllPaths
    public void AddError (ActionError error) => Errors.Add(error);

    /// <summary>
    /// Добавляет ошибки в список
    /// </summary>
    /// <param name="errors">Список ошибок</param>
    public void AddErrors(IEnumerable<ActionError> errors) => Errors.AddRange(errors);

    /// <summary>
    /// Добавление ошибки
    /// </summary>
    /// <param name="critical">Является ли ошибка критической</param>
    public void AddError (bool critical = true)
    {
        //Создаю ошибку
        ActionError error = new("", critical);

        //Добавляю ошибку
        AddError(error);
    }

    /// <summary>
    /// Добавление ошибки
    /// </summary>
    /// <param name="message">Сообщение об ошибке</param>
    /// <param name="critical">Является ли ошибка критической</param>
    public void AddError (string message, bool critical = true)
    {
        //Создаю ошибку
        ActionError error = new(message, critical);

        //Добавляю ошибку
        AddError(error);
    }

    /// <summary>
    /// Добавление ошибки
    /// </summary>
    /// <param name="errorObject">Объект ошибки</param>
    /// <param name="message">Сообщение об ошибке</param>
    /// /// <param name="critical">Является ли ошибка критической</param>
    public void AddError (string errorObject, string message, bool critical = true)
    {
        //Создаю ошибку
        ActionError error = new(errorObject, message, critical);

        //Добавляю ошибку
        AddError(error);
    }
    #endregion

    #region Добавление предупреждения
    /// <summary>
    /// Добавление предупреждения
    /// </summary>
    /// <param name="warning">Предупреждение</param>
    public void AddWarning (ActionWarning warning) => Warnings.Add(warning);

    /// <summary>
    /// Добавление предупреждений
    /// </summary>
    /// <param name="warnings">Список предупреждений</param>
    public void AddWarnings(IEnumerable<ActionWarning> warnings) => Warnings.AddRange(warnings);

    /// <summary>
    /// Добавление предупреждение
    /// </summary>
    /// <param name="message">Текст предупреждения</param>
    /// <param name="warningObject">Объект предупреждения</param>
    public void AddWarning (string message, string warningObject = "")
    {
        //Создаю предупреждение
        ActionWarning warning = new(warningObject, message);

        //Добавляю предупреждение
        AddWarning(warning);
    }
    #endregion

    #region Добавление информации
    /// <summary>
    /// Добавление информации
    /// </summary>
    /// <param name="info">Информация</param>
    public void AddInfo (ActionInfo info) => Info.Add(info);

    /// <summary>
    /// Добавление информации
    /// </summary>
    /// <param name="infos">Список информации</param>
    public void AddInfos (IEnumerable<ActionInfo> infos) => Info.AddRange(infos);

    /// <summary>
    /// Добавление информации
    /// </summary>
    /// <param name="message">Текст информации</param>
    /// <param name="infoObject">Объект информации</param>
    public void AddInfo (string message, string infoObject = "")
    {
        //Создаю информацию
        ActionInfo info = new(infoObject, message);

        //Добавляю информацию
        AddInfo(info);
    }
    #endregion

    #region Печать
    /// <summary>
    /// Печать списка ошибок
    /// </summary>
    /// <param name="formatList">Формат списка</param>
    /// <param name="formatItem">Формат элемента списка</param>
    /// <returns>Список ошибок в текстовой форме</returns>
    public string PrintErrorList (string formatList, string formatItem)
    {
        string elements =
#pragma warning disable CS8625
            Errors.Aggregate<IActionError, string>(null, (current, error) => current + error.PrintMessage(formatItem));
#pragma warning restore CS8625

        return string.Format(formatList, elements);
    }

    /// <summary>
    /// Печать списка предупреждений
    /// </summary>
    /// <param name="formatList">Формат списка</param>
    /// <param name="formatItem">Формат элемента списка</param>
    /// <returns>Список предупреждений в текстовой форме</returns>
    public string PrintWarningList (string formatList, string formatItem)
    {
        string elements =
#pragma warning disable CS8625
            Warnings.Aggregate<IActionWarning, string>(null,
#pragma warning restore CS8625
                (current, warning) => current + warning.PrintMessage(formatItem));

        return string.Format(formatList, elements);
    }

    /// <summary>
    /// Печать списка информации
    /// </summary>
    /// <param name="formatList">Формат списка</param>
    /// <param name="formatItem">Формат элемента списка</param>
    /// <returns>Список информации в текстовой форме</returns>
    public string PrintInfoList (string formatList, string formatItem)
    {
        string elements =
#pragma warning disable CS8625
            Info.Aggregate<IActionInfo, string>(null, (current, info) => current + info.PrintMessage(formatItem));
#pragma warning restore CS8625

        return string.Format(formatList, elements);
    }
    #endregion

    #region Проверка на наличие
    /// <summary>
    /// Проверяет, есть ли ошибки
    /// </summary>
    /// <param name="ignoreNonCritical">Игнорировать не критические</param>
    /// <returns>Наличие ошибок</returns>
    public bool HasErrors (bool ignoreNonCritical = false) =>
        ignoreNonCritical ? Errors.Any(static error => error.IsCritical) : Errors.Any();

    /// <summary>
    /// Проверяет, есть ли предупреждения
    /// </summary>
    /// <param name="ignoreInformWarning">Игнорировать информационные предупреждения</param>
    /// <returns>Наличие предупреждений</returns>
    public bool HasWarnings (bool ignoreInformWarning = false) => ignoreInformWarning
        ? Warnings.Any(static warning => !warning.IsInformWarning)
        : Warnings.Any();

    /// <summary>
    /// Проверяет, есть ли сообщения
    /// </summary>
    /// <param name="ignoreStatus">Игнорировать статусные сообщения</param>
    /// <returns>Наличие сообщений</returns>
    public bool HasInfo (bool ignoreStatus) => ignoreStatus ? Info.Any(static info => !info.IsStatusInfo) : Info.Any();

    /// <summary>
    /// Успешно ли завершилось
    /// </summary>
    public bool IsSuccess (bool ignoreNonCriticalErrors = false) =>
        !HasErrors(ignoreNonCriticalErrors) && !HasWarnings(true);
    #endregion

    #region Количество сообщений
    /// <summary>
    /// Количество ошибок
    /// </summary>
    /// <param name="ignoreNonCritical">Игнорировать не критические</param>
    /// <returns>Количество ошибок</returns>
    public int ErrorsCount (bool ignoreNonCritical = false) =>
        ignoreNonCritical ? Errors.Count(static error => error.IsCritical) : Errors.Count;

    /// <summary>
    /// Количество предупреждений
    /// </summary>
    /// <param name="ignoreInformWarning">Игнорировать информационные предупреждения</param>
    /// <returns>Количество предупреждений</returns>
    public int WarningsCount (bool ignoreInformWarning = false) => ignoreInformWarning
        ? Warnings.Count(static warning => !warning.IsInformWarning)
        : Warnings.Count;

    /// <summary>
    /// Количество информационных сообщений
    /// </summary>
    /// <param name="ignoreStatus">Игнорировать статусные сообщения</param>
    /// <returns>Количество информационных сообщений</returns>
    public int InfoCount (bool ignoreStatus) => ignoreStatus ? Info.Count(static info => !info.IsStatusInfo) : Info.Count;
    #endregion

    #region Добавление другого состояния

    /// <summary>
    /// Добавляет другое состояние (например, результат другого действия, который возвращает &lt;see cref="ActionState"/&gt;).
    /// </summary>
    /// <typeparam name="T">Тип параметра</typeparam>
    /// <param name="state">Запись состояния</param>
    public void AddState (ActionState<T> state)
    {
        AddErrors(state.Errors);
        AddWarnings(state.Warnings);
        AddInfos(state.Info);
    }
    #endregion
    #endregion
}