﻿using anbs_cp.Classes.Exceptions;
using anbs_cp.Enums;

namespace anbs_cp.Classes;

/// <summary>
/// Класс, добавляющий реализацию некоторых методов Delphi, которые упрощают работу в C#.
/// </summary>
public static class LikeDelphi
{
    /// <summary>
    /// Аналог функции IncludeTrailingBackslash
    /// </summary>
    /// <param name="path">Путь, к которому нужно добавить slash</param>
    /// <returns>Путь со slash в конце</returns>
    public static string IncludeTrailingBackslash (string path)
    {
        //По умолчанию сохраняем путь
        string result = path;

        //Если последний символ не "\", то добавим "\" в конце
        if (path[^1] != '\\')
            result = $@"{path}\";

        //Вернём результат
        return result;
    }

    /// <summary>
    /// Парсер строки в множество строк
    /// </summary>
    /// <param name="str">Строка, которую нужно разбить</param>
    /// <param name="delimiter">Символ-делитель строки</param>
    /// <returns>Массив строк</returns>
    public static List<string> ParseString (string str, char delimiter) => str.Split(delimiter).ToList();

    /// <summary>
    /// Переименовываю файл
    /// </summary>
    /// <param name="oldName">Старое имя файла (с полным путём)</param>
    /// <param name="newName">Новое имя файла (с полным путём)</param>
    /// <param name="ifExist">Что делать, если существует</param>
    /// <exception cref="RenameException">Если <paramref name="ifExist" /> имеет значение <value>RaiseException</value>, то генерируется ошибка переименования файла</exception>
    public static void RenameFile (string oldName, string newName,
        EOnExistAction ifExist = EOnExistAction.RaiseException)
    {
        //Если целевой файл существует
        if (File.Exists(newName))
            switch (ifExist)
            {
                //Возбуждать исключение
                case EOnExistAction.RaiseException:
                    throw new RenameException(newName, "Файл уже существует!");
                //Прерываю
                case EOnExistAction.Abort:
                    return;
                //Игнорирую и перезаписываю файл
                case EOnExistAction.Ignore:
                    break;
                //Только для папок (для файлов равносилен RaiseException)
                case EOnExistAction.RaiseExceptionIfNotEmpty:
                    throw new RenameException(newName, "Файл уже существует!");
                //Только для папок (для файлов равносилен Abort)
                case EOnExistAction.AbortIfNotEmpty:
                    return;
                //по умолчанию - RaiseException
                default:
                    throw new RenameException(newName, "Файл уже существует!");
            }

        //Перемещаю файл
        File.Move(oldName, newName, true);

        //Если начальный файл существует
        if (File.Exists(oldName))
            //- удаляю его
            File.Delete(oldName);
    }

    /// <summary>
    /// Удаление папки
    /// </summary>
    /// <param name="dir">Папка</param>
    public static void RemoveDir (string dir) => Directory.Delete(dir, true);

    /// <summary>
    /// Проверяет папку на пустоту
    /// Оригинальный ответ от OwenGlendower (https://www.cyberforum.ru/windows-forms/thread1995240.html)
    /// </summary>
    /// <param name="dir">Проверяемая папка</param>
    /// <returns>Пуста (<value>true</value>) или не пуста (<value>false</value>) папка <see cref="dir"/></returns>
    public static bool IsDirEmpty (string dir) => !Directory.EnumerateFiles(dir, "*.*", SearchOption.AllDirectories).Any();

    /// <summary>
    /// Переименовывает папку
    /// </summary>
    /// <param name="oldName">Старое имя папки (с полным путём)</param>
    /// <param name="newName">Новое имя папки (с полным путём)</param>
    /// <param name="ifExist">Что делать, если существует</param>
    /// <exception cref="RenameException">Если <paramref name="ifExist" /> имеет значение <value>RaiseException</value>, то генерируется ошибка переименования папки</exception>
    public static void RenameDir (string oldName, string newName,
        EOnExistAction ifExist = EOnExistAction.RaiseException)
    {
        //Если целевая папка существует
        if (Directory.Exists(newName))
        {
            switch (ifExist)
            {
                //Возбуждать исключение
                case EOnExistAction.RaiseException:
                    throw new RenameException(newName, "Папка уже существует!");
                //Прерывать
                case EOnExistAction.Abort:
                    return;
                //Игнорировать и перезаписывать папку
                case EOnExistAction.Ignore:
                    break;
                //Возбуждать исключение, если не пустая
                case EOnExistAction.RaiseExceptionIfNotEmpty:
                    if (!IsDirEmpty(newName))
                        throw new RenameException(newName, "Папка уже существует и не пуста!");

                    break;
                //Прерывать, если не пустая
                case EOnExistAction.AbortIfNotEmpty:
                    if (!IsDirEmpty(newName))
                        return;

                    break;
                //по умолчанию - RaiseException
                default:
                    throw new RenameException(newName, "Папка уже существует!");
            }

            //Удаляю целевую папку
            RemoveDir(newName);
        }

        //Перемещаю папку
        Directory.Move(oldName, newName);

        //Если начальная папка существует
        if (Directory.Exists(oldName))
            //- удаляю его
            RemoveDir(oldName);
    }
}