﻿using System.Transactions;

namespace anbs_cp.Database.Interfaces;

/// <summary>
/// Интерфейс для работы с базой данных
/// </summary>
public interface IDbEngine
{
    /// <summary>
    /// Строка подключения
    /// </summary>
    string ConnectionString { get; set; }

    #region Базовые операции

    /// <summary>
    /// Выполняем команду
    /// </summary>
    /// <param name="sql">Запрос</param>
    /// <param name="values">Данные запроса</param>
    /// <returns>Количество затронутых строк</returns>
    Task<int> ExecuteAsync (string sql, object? values = null);

    /// <summary>
    /// Выполняем команду
    /// </summary>
    /// <param name="sql">Запрос</param>
    /// <param name="values">Данные запроса</param>
    /// <returns>Количество затронутых строк</returns>
    int Execute (string sql, object? values = null);

    /// <summary>
    /// Запрос
    /// </summary>
    /// <typeparam name="T">Тип передаваемого значения</typeparam>
    /// <param name="sql">Запрос</param>
    /// <param name="values">Данные запроса</param>
    /// <returns>Возвращает массив типа <see cref="T"/> или значение по умолчанию</returns>
    Task<IEnumerable<T>> QueryAsync<T> (string sql, object? values = null);

    /// <summary>
    /// Запрос
    /// </summary>
    /// <typeparam name="T">Тип передаваемого значения</typeparam>
    /// <param name="sql">Запрос</param>
    /// <param name="values">Данные запроса</param>
    /// <returns>Возвращает массив типа <see cref="T"/> или значение по умолчанию</returns>
    IEnumerable<T> Query<T> (string sql, object? values = null);

    /// <summary>
    /// Запрос строки
    /// </summary>
    /// <typeparam name="T">Тип передаваемого значения</typeparam>
    /// <param name="sql">Запрос</param>
    /// <param name="values">Данные запроса</param>
    /// <returns>Возвращает массив типа <see cref="T"/> или значение по умолчанию</returns>
    Task<T?> QueryRowAsync<T> (string sql, object? values = null);

    /// <summary>
    /// Запрос строки
    /// </summary>
    /// <typeparam name="T">Тип передаваемого значения</typeparam>
    /// <param name="sql">Запрос</param>
    /// <param name="values">Данные запроса</param>
    /// <returns>Возвращает массив типа <see cref="T"/> или значение по умолчанию</returns>
    T? QueryRow<T> (string sql, object? values = null);
    #endregion

    #region Получение данных

    /// <summary>
    /// Получает массив данных (SELECT * FROM...)
    /// </summary>
    /// <typeparam name="T">Тип получаемых данных</typeparam>
    /// <param name="sql">SQL-запрос</param>
    /// <param name="values">Данные запроса</param>
    /// <returns>Массив результата запроса</returns>
    Task<IEnumerable<T>> GetResultsAsync<T> (string sql, object? values = null);

    /// <summary>
    /// Получает массив данных (SELECT * FROM...)
    /// </summary>
    /// <typeparam name="T">Тип получаемых данных</typeparam>
    /// <param name="sql">SQL-запрос</param>
    /// <param name="values">Данные запроса</param>
    /// <returns>Массив результата запроса</returns>
    IEnumerable<T> GetResults<T> (string sql, object? values = null);

    /// <summary>
    /// Получает строку в массиве данных
    /// </summary>
    /// <typeparam name="T">Тип получаемых данных</typeparam>
    /// <param name="sql">SQL-запрос</param>
    /// <param name="values">Данные запроса</param>
    /// <returns>Строки данных или null</returns>
    Task<T?> GetRowAsync<T> (string sql, object? values = null);

    /// <summary>
    /// Получает строку в массиве данных
    /// </summary>
    /// <typeparam name="T">Тип получаемых данных</typeparam>
    /// <param name="sql">SQL-запрос</param>
    /// <param name="values">Данные запроса</param>
    /// <returns>Строки данных или null</returns>
    T? GetRow<T> (string sql, object? values = null);

    /// <summary>
    /// Получает колонку в массиве данных
    /// </summary>
    /// <typeparam name="T">Тип получаемых данных</typeparam>
    /// <param name="sql">SQL-запрос</param>
    /// <param name="values">Данные запроса</param>
    /// <returns>Колонка данных или null</returns>
    Task<IEnumerable<T>> GetColAsync<T> (string sql, object? values = null) where T: IComparable, IConvertible, IEquatable<T>;

    /// <summary>
    /// Получает колонку в массиве данных
    /// </summary>
    /// <typeparam name="T">Тип получаемых данных</typeparam>
    /// <param name="sql">SQL-запрос</param>
    /// <param name="values">Данные запроса</param>
    /// <returns>Колонка данных или null</returns>
    IEnumerable<T> GetCol<T> (string sql, object? values = null) where T: IComparable, IConvertible, IEquatable<T>;

    /// <summary>
    /// Получение значение единичного поля
    /// </summary>
    /// <typeparam name="T">Тип получаемых данных</typeparam>
    /// <param name="sql">SQL-запрос</param>
    /// <param name="values">Данные запроса</param>
    /// <returns>Поле или null</returns>
    Task<T?> GetVarAsync<T> (string sql, object? values = null) where T: IComparable, IConvertible, IEquatable<T>;

    /// <summary>
    /// Получение значение единичного поля
    /// </summary>
    /// <typeparam name="T">Тип получаемых данных</typeparam>
    /// <param name="sql">SQL-запрос</param>
    /// <param name="values">Данные запроса</param>
    /// <returns>Поле или null</returns>
    T? GetVar<T> (string sql, object? values = null) where T: IComparable, IConvertible, IEquatable<T>;

    #endregion

    #region CRUD данные
    /// <summary>
    /// Вставляет данные в таблицу
    /// </summary>
    /// <typeparam name="T">Класс данных</typeparam>
    /// <param name="data">Данные</param>
    /// <param name="tableName">Имя таблицы</param>
    /// <returns>Результат выполнения</returns>
    Task<bool> InsertAsync<T> (T data, string tableName) where T: class;

    /// <summary>
    /// Вставляет данные в таблицу
    /// </summary>
    /// <typeparam name="T">Класс данных</typeparam>
    /// <param name="data">Данные</param>
    /// <param name="tableName">Имя таблицы</param>
    /// <returns>Результат выполнения</returns>
    bool Insert<T> (T data, string tableName) where T: class;

    /// <summary>
    /// Обновляет строку в таблице
    /// </summary>
    /// <typeparam name="T">Класс данных</typeparam>
    /// <param name="data">Данные</param>
    /// <param name="tableName">Имя таблицы</param>
    /// <param name="whereConditionColumn">Условие поиска строки. ВНИМАНИЕ! Должно быть одним из указанных свойств типа класса data</param>
    /// <returns>Результат выполнения</returns>
    Task<bool> UpdateAsync<T> (T data, string tableName, string whereConditionColumn) where T: class;

    /// <summary>
    /// Обновляет строку в таблице
    /// </summary>
    /// <typeparam name="T">Класс данных</typeparam>
    /// <param name="data">Данные</param>
    /// <param name="tableName">Имя таблицы</param>
    /// <param name="whereConditionColumn">Условие поиска строки. ВНИМАНИЕ! Должно быть одним из указанных свойств типа класса data</param>
    /// <returns>Результат выполнения</returns>
    bool Update<T> (T data, string tableName, string whereConditionColumn) where T: class;

    /// <summary>
    /// Удаляет строки
    /// </summary>
    /// <param name="data">Данные</param>
    /// <param name="tableName">Имя таблицы</param>
    /// <param name="whereConditionColumn">Условие поиска строки. ВНИМАНИЕ! Должно быть одним из указанных свойств типа класса data</param>
    /// <returns>Результат выполнения</returns>
    Task<bool> DeleteAsync<T> (T data, string tableName, string whereConditionColumn) where T: class;

    /// <summary>
    /// Удаляет строки
    /// </summary>
    /// <param name="data">Данные</param>
    /// <param name="tableName">Имя таблицы</param>
    /// <param name="whereConditionColumn">Условие поиска строки. ВНИМАНИЕ! Должно быть одним из указанных свойств типа класса data</param>
    /// <returns>Результат выполнения</returns>
    bool Delete<T> (T data, string tableName, string whereConditionColumn) where T: class;

    /// <summary>
    /// Удаляет строку
    /// </summary>
    /// <param name="data">Данные</param>
    /// <param name="tableName">Имя таблицы</param>
    /// <param name="whereConditionColumn">Условие поиска строки. ВНИМАНИЕ! Должно быть одним из указанных свойств типа класса data</param>
    /// <returns>Результат выполнения</returns>
    Task<bool> DeleteRowAsync <T> (T data, string tableName, string whereConditionColumn) where T: class;

    /// <summary>
    /// Удаляет строку
    /// </summary>
    /// <param name="data">Данные</param>
    /// <param name="tableName">Имя таблицы</param>
    /// <param name="whereConditionColumn">Условие поиска строки. ВНИМАНИЕ! Должно быть одним из указанных свойств типа класса data</param>
    /// <returns>Результат выполнения</returns>
    bool DeleteRow <T> (T data, string tableName, string whereConditionColumn) where T: class;
    #endregion
}